function [Vu,DGSM] = Vu_calculation_matlab_routins( k, N,samplemethod,differentiationmethod)

%%%%%%%input
%%%k = number of variables
%%%N = number of samples
%%%samplemethod == 1 means QMC sampling(Sobol)
%%%samplemethod == 0 means MCS sampling(Random)
%%%differentiationmethod == 1 means numerical differentiation
%%%differentiationmethod == 0 means automatic differentiation

%%%%%%%output
%%%Vu = matrix( 1,k )  output of Vu's  
%%%DGSM = matrix ( 1,k )  DGSM=Vu/D/pi^2 for uniformly distributed inputs

%%%%%%Reference
%%%I.M. Sobol, S. Kucherenko, Derivative based global sensitivity measures
%%%and their link with global sensitivity indices, Mathematics and Computers
%%%in Simulation, 79(10) (2009) 3009C3017.

    tic
    dfdx_value= zeros(N,k); 
    Vu=zeros(1,k);
    DGSA_t=zeros(1,k); 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
   if samplemethod==0
       %%%%%%Monte Carlo simulation used to generate the variable vector
       char_samplemethod='MC';
       X=unifrnd(0,1,N,k);%%%X=rand(N,k);
   end
   
   if samplemethod==1
       %%%%%QMC used to generate the Sobol variable vector in [0,1]
       char_samplemethod='QMC';
       geninit=1;
        xx = [];
        %%%%%for QMC integration
        genfirst = (geninit-1)*N+1;
        genlast = geninit*N;
            if (size(xx,1) < genlast)
                dum = zeros(genlast-size(xx,1)-1,k);
                for i = (size(xx,1)+1):genlast
                    dum(i-size(xx,1),:) = LPTAU51(i,k);
                end    
                xx = [xx; dum]; 
                dum = [];
            end
        X = xx(genfirst:genlast,1:k);%%%%%Samples of uniform(0 1)
   end

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%transform sample X in  [0,1] range to XX in [lower,upper] range
   range=zeros(2,k);
   [range]=load('range_ab.txt');
   lower=double(range(1,:));
   upper=double(range(2,:));
   
   for i=1:k
       XX(:,i)=lower(i)+(upper(i)-lower(i))*X(:,i);
   end
   
   y0=user_defined(XX');%%%%%% Function values at  X
   D=mean(y0.^2) - mean(y0).^2; %%%%D=var(y0);%%%%%% Function variance  
      
   if differentiationmethod==0  
   %%%%%%%%% AUTOMATICAL DIFFERENTIATION dfdx using symbols    
        char_differentiationmethod='automatical differentiation';
        temp_Variable=sym('x',[1,k]); 

        f=user_defined([temp_Variable]);%%%%The symbol of function
        for i=1:k
            dfdx(i)=diff(f,temp_Variable(i));  %%%%The symbol of dfdx
        end
       
        for i=1:N
            for j=1:k%%Obtain the value of dfdx at samples(dfdx=dfdu*dudx,where dudx=1/(upper-lower)) 
                dfdx_value(i,j)=subs(dfdx(j),temp_Variable,XX(i,:));
            end
        end
   end          
           
   if differentiationmethod==1
   %%%%%%%%% NUMERICAL DIFFERENTIATION dfdx=(f(x+delta)-f(x))/delta
       char_differentiationmethod='numerical differentiation';            
       for j=1:k      
            temp_point=XX; 
            delta=0.001*(upper(j)-lower(j));
            for i=1:N
               if XX(i,j)+delta<=upper(j)
                   temp_point(i,j)=XX(i,j)+delta;
                   delta_X(i)=delta;
               else
                   temp_point(i,j)=upper(j);
                   delta_X(i)=upper(j)-XX(i,j);
               end
           end
           temp_y=user_defined(temp_point');
           dfdx_value(:,j)=(temp_y-y0)./delta_X;   %%%%%dfdx  
       end   
   end
    %%%%%%%%%%%%%%%%%%Get the Vu by using QMC or MC samples
    for j=1:k
        Vu(j)= mean(dfdx_value(:,j).^2); %%%%%%%%%%%the output ...... Vu
        DGSM(j)=Vu(j)*(upper(j)-lower(j)).^2/D/pi/pi;%%%%%%%%%%%the output ...... DGSM
    end  

    toc    
end


